package MT::Plugin::MTNotifier;

use strict;
use warnings;
use base qw/MT::Plugin/;

use MT;
use MT::Plugin;
use MT::Util;
use MT::Util::UniqueID;
use MT::Sanitize;
use MT::I18N;
use MT::Log;
use Encode;

use constant _DEBUG => 0;

our $PLUGIN_NAME = 'MTNotifier';
our $VERSION = '1.0';

my $plugin = __PACKAGE__->new({
    name => $PLUGIN_NAME,
    id => lc($PLUGIN_NAME),
    key => lc($PLUGIN_NAME),
    version => $VERSION,
    author_name => 'Colsis,inc',
    author_link => 'https://www.colsis.jp/',
    description => '',
    l10n_class => $PLUGIN_NAME.'::L10N',
    system_config_template => \&system_config_tmpl,
    settings => new MT::PluginSettings([
        [lc($PLUGIN_NAME).'_plugin_enable', { Default => 0, Scope => 'system'}],
        [lc($PLUGIN_NAME).'_message', { Default => '', Scope => 'system'}],
        [lc($PLUGIN_NAME).'_start_date', { Default => '', Scope => 'system'}],
        [lc($PLUGIN_NAME).'_end_date', { Default => '', Scope => 'system'}],
        [lc($PLUGIN_NAME).'_target_dashboard', { Default => 0, Scope => 'system'}],
        [lc($PLUGIN_NAME).'_target_list_common', { Default => 0, Scope => 'system'}],
        [lc($PLUGIN_NAME).'_target_edit_entry', { Default => 0, Scope => 'system'}],
        [lc($PLUGIN_NAME).'_target_edit_content_data', { Default => 0, Scope => 'system'}],
        [lc($PLUGIN_NAME).'_timing', { Default => 0, Scope => 'system'}],
        [lc($PLUGIN_NAME).'_id', { Default => '', Scope => 'system'}],
    ]),
    registry => {
        callbacks => {
#            'set_notification_dashboard' => \&hdlr_set_notification_dashboard,
            'template_output.dashboard' => \&hdlr_tmpl_out,
            'template_output.list_common' => \&hdlr_tmpl_out,
            'template_output.edit_entry' => \&hdlr_tmpl_out,
            'template_output.edit_content_data' => \&hdlr_tmpl_out,
        },
    },
});

MT->add_plugin($plugin);

sub log {
    my ($self, $level, $message, $meta) = @_;
    return if (!_DEBUG && $level == MT::Log::DEBUG());
    MT->log({
        level => $level,
        message => $message,
        metadata => $meta,
        class => $PLUGIN_NAME,
    });
}

sub config {
    my ($self, $key, $blog_id) = @_;
    my $scope = $blog_id ? 'blog:'.$blog_id: 'system';
    return $self->get_config_value($key, $scope);
}

sub _get_targets {
    my ($self) = @_;
    my @targets = qw/dashboard list_common edit_entry edit_content_data/;

    my %targetmap = map({ $_ => $plugin->config(lc($PLUGIN_NAME).'_target_'.$_); } @targets);
    return grep({ $targetmap{$_} } keys(%targetmap));
}

sub system_config_tmpl {
    my ($self, $tmpl, $scope) = @_;

    return <<MTML;
<mtapp:setting
  id="@{[lc($PLUGIN_NAME)]}_plugin_enable"
  label="<__trans phrase="display notification">"
  hint="<__trans phrase="Controls the display of notifications. Select Show notifications and save to see the notification message.">"
  show_hint="1"
>
  <div class="custom-control custom-radio">
    <label>
      <input type="radio" class="" name="@{[lc($PLUGIN_NAME)]}_plugin_enable" value="1" <mt:if name="@{[lc($PLUGIN_NAME)]}_plugin_enable" eq="1">checked="checked"</mt:if>>
      <__trans phrase="notify">
    </label>
    <label>
      <input type="radio" class="" name="@{[lc($PLUGIN_NAME)]}_plugin_enable" value="0" <mt:unless name="@{[lc($PLUGIN_NAME)]}_plugin_enable">checked="checked"</mt:unless>>
      <__trans phrase="do not notify">
    </label>
  </div>
</mtapp:setting>

<mtapp:setting
  id="@{[lc($PLUGIN_NAME)]}_targets"
  label="<__trans phrase="target page">"
  hint="<__trans phrase="Select the target screen to display.">"
  show_hint="1"
>
  <div class="custom-control custom-checkbox">
    <label>
      <input type="checkbox" class="" name="@{[lc($PLUGIN_NAME)]}_target_dashboard" value="1" <mt:if name="@{[lc($PLUGIN_NAME)]}_target_dashboard">checked="checked"</mt:if>>
      <__trans phrase="dashboard">
    </label>
    <label>
      <input type="checkbox" class="" name="@{[lc($PLUGIN_NAME)]}_target_list_common" value="1" <mt:if name="@{[lc($PLUGIN_NAME)]}_target_list_common">checked="checked"</mt:if>>
      <__trans phrase="list common">
    </label>
    <label>
      <input type="checkbox" class="" name="@{[lc($PLUGIN_NAME)]}_target_edit_entry" value="1" <mt:if name="@{[lc($PLUGIN_NAME)]}_target_edit_entry">checked="checked"</mt:if>>
      <__trans phrase="edit entry">
    </label>
    <label>
      <input type="checkbox" class="" name="@{[lc($PLUGIN_NAME)]}_target_edit_content_data" value="1" <mt:if name="@{[lc($PLUGIN_NAME)]}_target_edit_content_data">checked="checked"</mt:if>>
      <__trans phrase="edit content data">
    </label>

  </div>
</mtapp:setting>

<mtapp:setting
  id="@{[lc($PLUGIN_NAME)]}_message"
  label="<__trans phrase="notify message">"
  hint="<__trans phrase="Enter the notification message.Text and URL are available.Character decoration cannot be set.">"
  show_hint="1"
>
  <div class="custom-control custom-input">
    <textarea rows="5" class="form-control" name="@{[lc($PLUGIN_NAME)]}_message"><mt:var name="@{[lc($PLUGIN_NAME)]}_message"></textarea>
  </div>
</mtapp:setting>

<mtapp:setting
  id="@{[lc($PLUGIN_NAME)]}_period"
  label="<__trans phrase="display period">"
  hint="<__trans phrase="The notification message is displayed only for the set period.">"
  show_hint="1"
>
  <div class="custom-control custom-input">
    <input type="text" class="date text-date" name="@{[lc($PLUGIN_NAME)]}_start_date" value="<mt:var name="@{[lc($PLUGIN_NAME)]}_start_date">">
     から 
    <input type="text" class="date text-date" name="@{[lc($PLUGIN_NAME)]}_end_date" value="<mt:var name="@{[lc($PLUGIN_NAME)]}_end_date">">
     まで
  </div>
</mtapp:setting>

<mtapp:setting
  id="@{[lc($PLUGIN_NAME)]}_"
  label="<__trans phrase="display timing">"
  hint="<__trans phrase="Select whether to display only once or per session.">"
  show_hint="1"
>
  <div class="custom-control custom-radio">
    <label>
      <input type="radio" class="" name="@{[lc($PLUGIN_NAME)]}_timing" value="0"<mt:unless name="@{[lc($PLUGIN_NAME)]}_timing"> checked="checked"</mt:unless>>
      <__trans phrase="once(use cookie)">
    </label>
    <label>
      <input type="radio" class="" name="@{[lc($PLUGIN_NAME)]}_timing" value="1"<mt:if name="@{[lc($PLUGIN_NAME)]}_timing"> checked="checked"</mt:if>>
      <__trans phrase="each session">
    </label>
  </div>
</mtapp:setting>

<mtapp:setting id="@{[lc($PLUGIN_NAME)]}_id">
  <input type="hidden" name="@{[lc($PLUGIN_NAME)]}_id" value="@{[MT::Util::UniqueID->create_uuid()]}">
</mtapp:setting>
MTML
}

#sub hdlr_set_notification_dashboard {
#    my ($cb, $messages) = @_;
#    return unless (_is_show());
#
#    ## error / warning / success
#    my $message = { level => 'warning' };
#    return unless ($plugin->config(lc($PLUGIN_NAME).'_message'));
#    $message->{text} = $plugin->config(lc($PLUGIN_NAME).'_message');
##    $message->{detail} = $plugin->config(lc($PLUGIN_NAME).'_link') if ($plugin->config(lc($PLUGIN_NAME).'_link'));
#
#    push(@{$messages}, $message);
#}

sub hdlr_tmpl_out {
    my ($cb, $app, $out, $param, $tmpl) = @_;

    my @targets = _get_targets();
    (my $target = $param->{template_filename}) =~ s/^[^\/]*\///;
    my $type = $app->param('_type');
#    my $blog_id = $app->param('blog_id');

    return unless (_is_show());
    return unless (grep({ $_ eq $target } @targets));
    return unless ($$out =~ /<\/html>/msxi);
    return if ($$out =~ /id="@{[lc($PLUGIN_NAME)]}-modal"/msxi);

    #### list_common だと asset とかでも出てしまう問題の対策
    if ($target eq 'list_common') {
        return unless ($type =~ /^(?:content_data|entry|page)$/);
    }

    if ($target eq 'dashboard') {
        my $alert = <<HTML;
<div class="alert alert-danger" role="alert">
@{[
  Encode::decode('utf8', $plugin->translate('notification from administrator.')).
  '('.
  '<a href="#" onclick="'.lc($PLUGIN_NAME).'OpenModal(); return false;" id="'.lc($PLUGIN_NAME).'-open-modal">'.
  Encode::decode('utf8', $plugin->translate('check')).
  '</a>'.
  ')'
]}
</div>
HTML
        $$out =~ s{<form\s+name="dashoboard_reload_form"[^>]+>.*?</form>\s+</div>\s+</div>}{$&$/$alert$/}msxig;
    }

    my $modal_js = _generate_modal_js();
    my $modal_html = _generate_modal_html();
    $$out =~ s{</body>}{$modal_js$modal_html$&}msxi;

    if ($plugin->config(lc($PLUGIN_NAME).'_timing')) {
        ## session毎
        return if ($app->session(lc($PLUGIN_NAME)));
    } else {
        ## cookie
        return if ($app->cookie_val(lc($PLUGIN_NAME)) eq $plugin->config(lc($PLUGIN_NAME).'_id'));
    }

    my $modal_open_js = _generate_modal_open_js();
    $$out =~ s{</body>}{$&$/$modal_open_js}msxi;


    unless ($plugin->config(lc($PLUGIN_NAME).'_timing')) {
        ## 一度表示したら2週間表示しない
        $app->bake_cookie(
            -name => lc($PLUGIN_NAME),
            -value => $plugin->config(lc($PLUGIN_NAME).'_id'),
            -expires => '+14d',
        );
    } else {
        $app->session(lc($PLUGIN_NAME), 1);
    }

}

sub _generate_modal_html {
    my $message = $plugin->config(lc($PLUGIN_NAME).'_message');
    $message = MT::Util::encode_html($message);
    $message =~ s{\r\n|\r|\n}{<br>}msxig;
    $message =~ s{https?://[^\s]+(?:/|html?)}{<a href="$&" target="_blank">$&</a>}msxig;

    my $modal = <<MODAL;
<style>

div.modal-header {
  background: #FFF;
  border-top: solid 8px #cc3535;
  padding: 10px 0 30px 30px;
  font-family: ui-sans-serif;
}

div.modal-header > h3 {
  color: #cc3535;
}

div.modal-body {
  padding: 0 30px;
  font-family: ui-sans-serif;
} 

\@media (min-width: 768px) {
  div.modal-dialog {
    max-width: 45%;
  }
}

</style>
<div class="modal fade" role="dialog" id="@{[lc($PLUGIN_NAME)]}-modal" aria-labelledby="@{[lc($PLUGIN_NAME)]}-modal-label" aria-hidden="true">
  <div class="modal-dialog" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <h3>@{[Encode::decode('utf8', $plugin->translate('notification'))]}</h3>
<!--
        <h4 class="modal-title" id="@{[lc($PLUGIN_NAME)]}-modal-label">@{[$plugin->translate('notification')]}</h4>
        <button type="button" class="close" data-dismiss="modal" aria-label="Close">
          <span aria-hidden="true">&times;</span>
        </button>
-->
      </div>
      <div class="modal-body">${message}</div>
      <div class="modal-footer">
        <button type="button" class="btn btn-primary" data-dismiss="modal">OK</button>
      </div>
    </div>
  </div>
</div>
MODAL

    return $modal.$/;
}

sub _generate_modal_js {
    my $js = <<js;
<script type="text/javascript">
function @{[lc($PLUGIN_NAME)]}OpenModal () {
  jQuery('#@{[lc($PLUGIN_NAME)]}-modal').modal('show');
  return false;
}
</script>
js
    return $js.$/;
}

sub _generate_modal_open_js {
    my $js = <<js;
<script type="text/javascript">
(function (\$) { @{[lc($PLUGIN_NAME)]}OpenModal(); })(jQuery);
</script>
js
    return $js.$/;
}

sub _is_show {
    return 0 unless ($plugin->config(lc($PLUGIN_NAME).'_plugin_enable'));

    my ($d, $m, $y) = (localtime(time))[3..5];
    $y += 1900;
    $m++;
    $m = $m < 10 ? '0'.$m: $m;
    $d = $d < 10 ? '0'.$d: $d;
    my $today = $y.$m.$d;

    my $start = $plugin->config(lc($PLUGIN_NAME).'_start_date');
    (my $st_y) = $start =~ /^(\d{4})/;
    (my $st_m) = $start =~ /-(\d{2})-/;
    (my $st_d) = $start =~ /-(\d{2})$/;
    my $st = $st_y.$st_m.$st_d;
    
    my $end = $plugin->config(lc($PLUGIN_NAME).'_end_date');
    (my $ed_y) = $end =~ /^(\d{4})/;
    (my $ed_m) = $end =~ /-(\d{2})-/;
    (my $ed_d) = $end =~ /-(\d{2})$/;
    my $ed = $ed_y.$ed_m.$ed_d;

    return 1 if (!$st && !$ed);
    return 1 if ($today >= $st && !$ed);
    return 1 if (!$st && $today <= $ed);
    return ($today >= $st && $today <= $ed);
}



package MTNotifier::L10N;
use strict;
use base 'MT::Plugin::L10N';


package MTNotifier::L10N::en_us;
use strict;
use base 'MTNotifier::L10N';
use vars qw(%Lexicon);
%Lexicon = ();


package MTNotifier::L10N::ja;
use strict;
use base 'MTNotifier::L10N';
use vars qw(%Lexicon);
%Lexicon = (
#    'use this plugin' => 'このプラグインを使う',
    'display notification' => '通知の表示',
    'notify' => '通知を表示する',
    'do not notify' => '通知を表示しない',
    'message link url' => 'メッセージのリンクURL',
    'notify message' => '通知メッセージ本文',
    'display period' => '表示期間',
    'notification' => 'お知らせ',
    'target page' => '表示対象画面',
    'dashboard' => 'ダッシュボード',
    'list common' => '記事・ウェブページ・コンテンツデータ一覧',
    'edit entry' => '記事・ウェブページ編集',
    'edit content data' => 'コンテンツデータ編集',
    'notification from administrator.' => '管理者からのお知らせがあります。',
    'check' => '確認する',
    'display timing' => '表示タイミング',
    'once(use cookie)' => '一度のみ(Cookie利用)',
    'each session' => 'セッション毎に都度表示',
    'Controls the display of notifications. Select Show notifications and save to see the notification message.' => '通知の表示を制御します。「通知を表示する」を選択し、保存すると通知メッセージが表示されます。',
    'Select the target screen to display.' => '表示する対象画面を選択します。',
    'Enter the notification message.Text and URL are available.Character decoration cannot be set.' => '通知メッセージを入力します。テキストとURLが利用できます。文字の装飾は設定できません。',
    'The notification message is displayed only for the set period.' => '設定された期間だけ通知メッセージが表示されます。',
    'Select whether to display only once or per session.' => '一度のみ表示するか、セッション毎に表示するかを選択します',   
);

1;
__END__
